-- Dusk Driver skin
-- Dylan Fitterer and Martin Nerurkar

-- First, some general Utility things

-- Find out what quality level the game is set to run at so we can scale details as needed
quality = GetQualityLevel4()
hifi = quality > 2
lofi = quality < 2

function ifhifi(if_true, if_false) -- create this function to help when changing a setting when we're in High or Ultra quality mode
  if hifi then return if_true else return if_false end
end
function iflofi(if_true, if_false) -- create this function to help when changing a setting when we're in High or Ultra quality mode
  if lofi then return if_true else return if_false end
end

-- Find out if we're in a wakeboard mode or a vehicle mode
jumping = PlayerCanJump()

function ifwakeboard(if_true, if_false) -- create this function to help when changing a setting when we're in wakeboard mode
	if jumping then return if_true else return if_false end
end


-- Get the track and skin properties
skinvars = GetSkinProperties()
trackWidth = skinvars["trackwidth"]
ispuzzle = skinvars.colorcount>1


-- configure some of the game's built-in effects and objects
SetScene{
	glowpasses = 0, -- no glow effect needed
	glowspread = 0, -- no glow effect needed
	radialblur_strength = 0, -- no radial blur effect needed
	dynamicFOV = false, -- no dynamic field of view effect needed
	airdebris_count = 0, -- no decorative air particles needed
	minimap_colormode = "track", -- color the minimap the normal way
	
	water = ifwakeboard(true, false), -- use the built-in digital water effect only in wakeboard modes
		towropes = ifwakeboard(true, false), -- use the towropes if we're in wakeboard mode
		watertint = {r=255,g=255,b=255,a=234},
		minwaterrandomdarkness = 0.95,
		watertype = 2,
		watertexture = "WaterCubesOutside.png",--texture used to color the dynamic "digital water" surface
		numvisiblewaterchunks = 8		
}


-- Get Album Art pallete and overwrite defeault Track colors if we get enough colors
trackPalette = GetAlbumArtPalette{}

-- Check for Greyscale
maxGreyDeviation = 0
for i=1, 2 do
	local c = trackPalette[i]
	local avg = (c.r + c.g + c.b) / 3.0
	local biggestGreyDev = math.max(math.abs(c.b-avg), math.max(math.abs(c.r-avg), math.abs(c.g-avg)))
	maxGreyDeviation = math.max(maxGreyDeviation, biggestGreyDev)
end

-- this track is too colorless (maybe from greyscale album art). 
if maxGreyDeviation < .075 then 
	
	-- The default color pallete (from light orange to deep red)
	SetTrackColors{
		{r=250, g=150, b=0},
		{r=250, g=70, b=0},
		{r=250, g=0, b=10}
	}
	
else

	if #trackPalette >= 2 then

		selectedTrackColors = {}
		selectedTrackColors[1] = trackPalette[1]
		selectedTrackColors[2] = trackPalette[1] -- this will be blended 
		selectedTrackColors[3] = trackPalette[2]
		
		-- Increase Saturation
		-- public-domain function by Darel Rex Finley
		-- http://alienryderflex.com/saturation.html
		Pr = .299
		Pg = .587
		Pb = .114

		for i=1, #selectedTrackColors do
			local maxRGBvalue = math.max(selectedTrackColors[i].r, selectedTrackColors[i].g, selectedTrackColors[i].b)
			if (maxRGBvalue < 0.8) then
				R = selectedTrackColors[i].r
				G = selectedTrackColors[i].g
				B = selectedTrackColors[i].b
				change = 1 + 0.8 / maxRGBvalue -- amount of saturation increase

				P = math.sqrt( R*R*Pr + G*G*Pg+ B*B*Pb ) 
				
				R = P+(R-P) * change
				G = P+(G-P) * change
				B = P+(B-P) * change
				
				selectedTrackColors[i].r = R
				selectedTrackColors[i].g = G
				selectedTrackColors[i].b = B		
			end
		end
		
		-- Create in-between color
		selectedTrackColors[2].r = ( math.max(selectedTrackColors[1].r, selectedTrackColors[3].r) - math.min(selectedTrackColors[1].r, selectedTrackColors[3].r) ) / 2 + math.min(selectedTrackColors[1].r, selectedTrackColors[3].r)
		selectedTrackColors[2].g = ( math.max(selectedTrackColors[1].g, selectedTrackColors[3].g) - math.min(selectedTrackColors[1].g, selectedTrackColors[3].g) ) / 2 + math.min(selectedTrackColors[1].g, selectedTrackColors[3].g)
		selectedTrackColors[2].b = ( math.max(selectedTrackColors[1].b, selectedTrackColors[3].b) - math.min(selectedTrackColors[1].b, selectedTrackColors[3].b) ) / 2 + math.min(selectedTrackColors[1].b, selectedTrackColors[3].b)
			
		SetTrackColors(selectedTrackColors)
		
	end

end


-- in puzzle modes (eraser, vegas, pusher, and pointmain), we also need to set the blocks' colors
if ispuzzle then 
	if skinvars.colorcount < 5 then
		blockColors ={
			{r=126, g=0, b=255}, -- purple
		   {r=0, g=66, b=240}, -- blue
		   {r=255, g=189, b=15}, -- yelloworange
			{r=227, g=38, b=0} -- red
		}
	else
		blockColors ={
			{r=126, g=0, b=255}, -- purple
		   {r=0, g=66, b=240}, -- blue
		   {r=0, g=184, b=0}, -- green
		   {r=255, g=189, b=15}, -- yelloworange
			{r=227, g=38, b=0} -- red
		}
	end
	SetBlockColors(blockColors)	
end


-- We use a simple sky-sphere shader here, that automatically colors with the highway colors
SetSkybox{
	--skysphere = "sky.png",
	skysphere = BuildMaterial {
		shader = "VertexColorUnlitTinted",
		shadercolors={
			_Color = { colorsource="highway" }
		},
		textures={
			_MainTex = "sky.png"
		}
	},
	--skyscreen = "sky.shader",
	--texture = "skyline.jpg",
	color = {255,255,255,0}
}


-- set up the look of the 2D rings the game places at intense sections of the track
SetRings{
	texture="SimpleRing.png",
	shader="VertexColorUnlitTintedAlpha",
	layer=13,
	
	size=0, -- Scale 0 because we want no regular rings
	percentringed=0.0,	
	
	-- Data for air rings
	airtexture="SimpleRing.png",
	airshader="VertexColorUnlitTintedAlpha",
	airsize=16 
}

-- Setup our Player Vehicle
if not jumping then 

	-- VEHICLE MODE.
	-- Configure the camera, vehicle and blocks for vehicle modes
		
	SetPlayer{	
		-- Configure the camera
		cameramode = "third",
		camfirst={
			pos={0,1.84,-0.8},
			rot={0,0,0}},
		--sets the two camera positions for 3rd person mode. lerps out towards pos2 when the song is less intense
		camthird={
			pos={0,2,-1.5},
			rot={15,0,0},
			strafefactorFar = .75,
			pos2={0,3.5,-3.5},
			rot2={17,0,0},
			strafefactorFar = 1,
			transitionspeed = 1,
			puzzleoffset=-0.65,
			puzzleoffset2=-1.5},
		-- Configure the vehicle
		vehicle={
			min_hover_height= 0.0,
			max_hover_height = 0.0,
			use_water_rooster = false,
			smooth_tilting = false,
			--smooth_tilting_speed = 50,
			--smooth_tilting_max_offset = 30,
			pos={x=0,y=0,z=0},
			mesh = BuildMesh{
				mesh="vehicle_car.obj"
			},
			colorMode="static",
			material = BuildMaterial{
				renderqueue = 2000,
				shader = "UnlitTintedTexEmission.shader",
				shadercolors={
					_Color={colorsource="highway" }
				},
				textures={
					_MainTex = "vehicle.png",
					_EmissionTex = "vehicle_emission.png"
				}
			},
			layer = 15,
			thrusters = {
				crossSectionShape={
					{-.35,-.35,0},
					{-.5,0,0},
					
					{-.35,.35,0},
					{0,.5,0},
					
					{.35,.35,0},
					{.5,0,0},
					
					{.35,-.35,0}
				},
				perShapeNodeColorScalers={.5,1,1,1,1,1,.5},
				material = BuildMaterial{
					renderqueue = 3000,
					shader="VertexColorUnlitTinted",
					shadercolors={
						_Color = fif(forceStaticColorOntoMonoBlocks, overrideHighwayColor, {255,255,255})
					}
				},
				layer = 15,
				colorscaler = {close=2.5, far=0},
				sizefalloff = 1,
				minrenderedsize = 0.05,
				extrusions=25,
				stretch=-0.12,
				updateseconds = 0.025,
				instances={
					{pos={-0.21,0.34,-2},rot={0,0,0},scale={0.7,0.4,0.5}}, -- left thruster
					{pos={0.21,0.34,-2},rot={0,0,0},scale={0.7,0.4,0.5}}, -- right thruster
				}
			}
		}
	}

	-- end VEHICLE MODE.
else

	-- WAKEBOARD MODE.
	-- Configure the camera and surfer humanoid for a wakeboard mode

	SetPlayer{
		cameramode = "first_jumpthird",

		camfirst={
			pos={0,2.7,-3.50475},
			rot={20.49113,0,0},
			strafefactor = 1
		},
		--sets the two camera positions for 3rd person mode. lerps out towards pos2 when the song is less intense
		camthird={ 
			pos = {0,2.7,-3.50475},
			rot = {20,0,0},
			strafefactor = 0.75,
			pos2 = {0,2.8,-3.50475},
			rot2 = {20.49113,0,0},
			strafefactorFar = 1},
		surfer={
			arms={
				material = BuildMaterial{
					shader="VertexColorUnlitTinted",
					shadercolors={
						_Color={colorsource="highwayinverted"}
					},
					texture="wakeboarderTexture.png"
				},
				shadowcaster = false,
				shadowreceiver = false,
				recalculateNormals = false
			},
			body={
				material = BuildMaterial{
					shader = "VertexColorUnlitTinted",
					shadercolors={
						_Color={colorsource="highwayinverted"}
					},
					textures={
						_MainTex = "wakeboarderAO.png",
					}
				},
				shadowcaster = false,
				shadowreceiver = false
			}
		}
	}

	-- end WAKEBOARD MODE.
end

-- Materials for the block grid beneath the vehicle
SetPuzzleGraphics{
	usesublayerclone = false,
	puzzlematchmaterial = {shader="Unlit/Transparent",texture="tileMatchingBars.png",aniso=9},
	puzzleflyupmaterial = {shader="VertexColorUnlitTintedAddFlyup",texture="blockPickup.png"}, -- was tileMatchingBars
	puzzlematerial = {shader="VertexColorUnlitTintedAlpha",texture="tilesSquare.png",texturewrap="clamp",aniso=9, usemipmaps="false",shadercolors={_Color={255,255,255,255}}}
}


blockMaterial = BuildMaterial{	
	shader = "UnlitTintedTexDistanceDarken.shader",
	textures={
		_MainTex = "objects.png",
	},
	shadersettings={
		_StartDistance=3, 
		_FullDistance=20,
		_MaxDarkness=.35
	}
},

-- Define the items on the track
SetBlocks{
	maxvisiblecount = ifhifi(100, 70),
	colorblocks={ -- ordinary blocks to collect
		mesh = "bevelblock.obj",
		shader = "UnlitTintedTexDistanceDarken.shader",
		texture = "bevelblock.png",
		shadersettings={
			_StartDistance=10, 
			_FullDistance=80,
			_MaxDarkness=.4
		},
		float_on_water = false,
		scale = {1,1,1}
	},
	greyblocks={ -- spike blocks to avoid
		mesh = "bevelblockSpikes.obj",
		shader = "UnlitTintedTexDistanceDarken.shader",
		texture = "bevelblock.png",
		shadersettings={
			_StartDistance=10, 
			_FullDistance=80,
			_MaxDarkness=.4
		},
		float_on_water = false
	},
	powerups={
		powerpellet={ -- the power block inside corkscrew loops on mono mode
			mesh = "clusterBlock.obj",
			shader = "UnlitTintedTex",
			texture = "clusterBlock.png"
		},
		whiteblock={ -- the white blocks found in puzzle modes				
			mesh = "bevelblock.obj",
			shader = "UnlitTintedTex", -- Shouldn't darken with distance to look like it glows
			texture = "bevelblock.png",
			shadersettings={
				_StartDistance=10, 
				_FullDistance=80,
				_MaxDarkness=.4
			}
		}
	}
}

SetBlockFlashes{
	--texture
	--sizescaler
	sizescaler_missed = 0
}

-- Setup some tables which we use throughout the skin

track = GetTrack()--get the track data from the game engine


--detect when the track is in a loop or corkscrew
--note, some modes may already have set the "funkyrot" field
for i=1,#track do
	local roll = track[i].roll % 360
	local tilt = track[i].tilt % 360
	local absTilt = math.abs(tilt)
	local suspiciousRotation = false
	if roll > 5 or roll < -5 or (absTilt > 80 and absTilt < 260) then
		suspiciousRotation = true
	end

	if track[i].funkyrot or suspiciousRotation then
		track[i].funkyrot = true -- in case the current mode isn't setting funkyrot, we figured it out here
	else
		track[i].funkyrot = false
	end
end


hasSuspensionBridge = false
hasSkyscrapers = false
suspensionBridgeNodes = {}
suspensionBridgePos = 0
SkyscraperSegments = {}
prevWasFunkyRot = true -- this is true to track the first segment

do
	local i = 1
	local length = 0;
	local averageIntensity = 0
	local averageTilt = 0
	local averagePan = 0
	local totalSegmentNum = 0
	local currentSegmentNum = 0
	
	-- Get us the amount of total non-funnkyrot-segments	
	for i=1, #track do
		if not track[i].funkyrot then		
			if prevWasFunkyRot then
				prevWasFunkyRot = false
				totalSegmentNum = totalSegmentNum + 1
			end
		else
			prevWasFunkyRot = true
		end
	end
	
	prevWasFunkyRot = true
	
	for i=1, #track do
		suspensionBridgeNodes[i] = 0
		SkyscraperSegments[i] = 0
	
		if not track[i].funkyrot then		
			if prevWasFunkyRot then
				prevWasFunkyRot = false
				currentSegmentNum = currentSegmentNum + 1
			end
			length = length + 1
			averageIntensity = averageIntensity + track[i].intensity
			averageTilt = averageTilt + math.abs(track[i].tilt % 360)
			averagePan = averagePan + math.abs(track[i].pan % 360)
			
		else
			if not prevWasFunkyRot then
				prevWasFunkyRot = true;
				
				averageIntensity = averageIntensity / length
				averageTilt = averageTilt / length		
				averagePan = averagePan / length				
								
				if ( currentSegmentNum > 1 and currentSegmentNum < totalSegmentNum ) then
				
					-- Do we have a fitting segment for a bridge?
					if ( not hasSuspensionBridge and averageIntensity < 0.6 and averagePan < 200 and averageTilt > 60 and #skinvars["lanedividers"] <= 2 and not jumping ) then
						-- track back and mark these sections as suspension bridge sections
						-- also mark these sections as funkyrot to make the track shrink nicely
						
						local bridgeLength = 300
						
						if length > (bridgeLength * 3) then
												
							suspensionBridgePos = i - 450 - 150 - 25 -- half length of miniumsegment length, half length of bridge, half length of -z value of bridge mesh
							
							-- the -50 / +50 is to give us a little wiggle room for the suspensionBridgeNodes to supress the guardrails
							for k = -60, (bridgeLength + 30 + 50) do
								currentNode = i - ( 450 ) - ( 150 ) + k 							
								suspensionBridgeNodes[currentNode] = 1
								
								-- only designate the actual bridge position as funkyrot though
								if ( k > 0 and k <= bridgeLength) then
									track[currentNode].funkyrot = true
								end
							end					
						
							print("SuspensionBridge on Segment " .. currentSegmentNum )
							hasSuspensionBridge = true
						
						end
					
					-- Do we have a fitting segment for the Skyscrapers
					elseif ( not hasSkyscrapers and currentSegmentNum > totalSegmentNum / 2 and averageIntensity >= 0.6) then
						-- track back and mark these sections as Skyscraper Sections
					
						if length < 800 then			
						
							for k = 0, length do
								SkyscraperSegments[i-length+k] = 1
							end
							
							print("Skyscrapers on Segment " .. currentSegmentNum )
							hasSkyscrapers = true;
							
						end
					end
				end	
				averageIntensity = 0;
				averageRoll = 0
				averageTilt = 0		
				averagePan = 0
				length = 0
			end		
		end	
	end
	--print("Last Regular Segment Length is " .. length)
end
	


-- fix the funkyrot to start a bit earlier before loops
do
	prevWasNormal = true
	local i = 1
	while i<=#track do
		if track[i].funkyrot then		
			if prevWasNormal then
				if i > 60 then
					for j=i-60,i do
						track[j].funkyrot = true
					end
				end
			end

			prevWasNormal = false
		else -- and also to continue a ways further past loops
			if not prevWasNormal then
				if #track > i + 60 then
					local bound = i + 60
					local k = i
					while k<bound do
						track[k].funkyrot = true
						k = k + 1
					end
					i = k
				end
			end

			prevWasNormal = true
		end

		i = i + 1
	end
end

-- Scales that scale down for funky-nodes (for use in repeated rails)
onlyRegularRotScalesSoft = {}
onlyRegularRotScalesSoftX = {}
onlyRegularRotScalesSoftY = {}

onlyRegularRotScales = {}
onlyFunkyRotScales = {}

softScaleSteps = 60
prevWasNormal = true
for i=1,#track do
	if track[i].funkyrot then	
		
		if prevWasNormal then
			prevWasNormal = false
		
			if i > softScaleSteps then
				local k = 0
				-- gradually reduce the scale before the continuous mesh is interrupted for the funkyzone
				for j=i-softScaleSteps,i do 
					onlyRegularRotScalesSoft[j] = {1-k/softScaleSteps,1-k/softScaleSteps,1}
					onlyRegularRotScalesSoftX[j] = {1-k/softScaleSteps,1,1}
					onlyRegularRotScalesSoftY[j] = {1,1-k/softScaleSteps,1}
					
					onlyRegularRotScales[j] = {x=0, y=0, z=0}
					onlyFunkyRotScales[j] = {x=0, y=0, z=0}
				
					k = k + 1
				end
			end
		else
			onlyRegularRotScalesSoft[i] = {0,0,0}
			onlyRegularRotScalesSoftX[i] = {0,0,0}
			onlyRegularRotScalesSoftY[i] = {0,0,0}
			onlyRegularRotScales[i] = {x=0, y=0, z=0}
			onlyFunkyRotScales[i] = {x=1, y=1, z=1}
		end
		
	else
	
		if not prevWasNormal then
			prevWasNormal = true
			
			if i > softScaleSteps then
				local k = 0
				-- gradually reduce the scale after the continuous mesh was interrupted for the funkyzone
				for j=i-softScaleSteps,i do 
					onlyRegularRotScalesSoft[j] = {k/softScaleSteps,k/softScaleSteps,1}
					onlyRegularRotScalesSoftX[j] = {k/softScaleSteps,1,1}
					onlyRegularRotScalesSoftY[j] = {1,k/softScaleSteps,1}

					onlyRegularRotScales[j] = {x=0, y=0, z=0}
					onlyFunkyRotScales[j] = {x=0, y=0, z=0}

					k = k + 1
				end
			end
		else
			onlyRegularRotScalesSoft[i] = {1,1,1}
			onlyRegularRotScalesSoftX[i] = {1,1,1}
			onlyRegularRotScalesSoftY[i] = {1,1,1}
			onlyRegularRotScales[i] = {x=1, y=1, z=1}
			onlyFunkyRotScales[i] = {x=0, y=0, z=0}
		end
		
	end	
end



-- BUILD RAILS


groundMaterial = BuildMaterial {
	shader = "UnlitTintedTex",
	shadercolors={
		_Color = { colorsource="highway" }
	},
	textures={
		_MainTex = "ground.png"
	}
}

roadmarksMaterial = BuildMaterial {
	shader = "UnlitTintedTex",
	shadercolors={
		_Color = { colorsource="highway" }
	},
	textures={
		_MainTex = "roadmarks.png"
	}
}

CreateRail{ 
	-- add a road surface for the vehicle to drive on, including small banks
	positionOffset={
		x=0,
		y=ifwakeboard(-4,0)
	},
	crossSectionShape={
		{x=-trackWidth-6,y=-2},
		{x=-trackWidth,y=0},
		{x=trackWidth,y=0},
		{x=trackWidth+6,y=-2}
	},
	wrapnodeshape = true,
	fullfuture = true, 
	flatten= false,

	colorMode="static",
	material = groundMaterial,
	
	shadowreceiver = false,
	shadowcaster = false,
	calculatenormals = false
}


CreateRail{ 
	-- This one goes down way below the road to create more contrasting darkness in the distance
	positionOffset={
		x=0,
		y=0
	},
	crossSectionShape={
		{x=trackWidth+100,y=-2},
		{x=trackWidth+100,y=-100},
		{x=-trackWidth-100,y=-100},
		{x=-trackWidth-100,y=-2}
	},
	wrapnodeshape = true,
	fullfuture = true, 
	flatten=false,
	stretch = 3,		
	
	colorMode="static",
	material = groundMaterial,
	
	shadowreceiver = false,
	shadowcaster = false,
	calculatenormals = false,
	scales = onlyRegularRotScalesSoft -- so it shrinks when approaching a funkyrot
}


-- Draw the lane lines
if not jumping then
	local laneDividers = skinvars["lanedividers"]
	for i=1,#laneDividers do
		CreateRail{
			positionOffset={
				x=laneDividers[i],
				y=0.02},
			crossSectionShape={
				{x=-.04,y=0},
				{x=.04,y=0}},
			flatten=false,
			nodeskip = 2,
			wrapnodeshape = false,
			fullfuture = iflofi(false, true), 
					
			colorMode="static",
			material = roadmarksMaterial,
			
			-- textureMode="repeataround",
			-- texture="roadmarks.png",
			-- shader = "VertexColorUnlitTinted",
			-- shadercolors={
			-- 	_Color = { colorsource="highway" }
			-- },
		}
	end

	local shoulderLines = skinvars["shoulderlines"]
	for i=1,#shoulderLines do
		CreateRail{
			positionOffset={
				x=shoulderLines[i],
				y=0.02},
			crossSectionShape={
				{x=-.1,y=0},
				{x=.1,y=0}},
			flatten=false,
			--nodeskip = 2,
			wrapnodeshape = false,
			fullfuture = iflofi(false, true), 
					
			colorMode="static",
			material = roadmarksMaterial,
			
			-- textureMode="repeataround",
			-- texture="roadmarks.png",
			-- shader = "VertexColorUnlitTinted",
			-- shadercolors={
			-- 	_Color = { colorsource="highway" }
			-- },
		}
	end
end



----------------------------------------------------
-- DECORATIVE OBJECTS

----------------------------------------------------
-- Batch Rendered Objects
-- These objects are only shown occasionally
-- Most are only shown when not in wakeboard mode

-- Build Materials


objectMaterial = BuildMaterial {
	shader = "UnlitTintedTex",
	shadercolors={
		_Color = { colorsource="highway" }
	},
	textures={
		_MainTex = "objects.png"
	}
}

lightMaterial = BuildMaterial {
	shader = "UnlitTintedTexEmissionDistanceDarken.shader",
	shadercolors={
		_Color = { colorsource="highway" }
	},
	textures={
		_MainTex = "objects.png",
		_EmissionTex = "objects_lights.png"
	},
	shadersettings={
		_StartDistance=85, 
		_FullDistance=850,
		_MaxDarkness=0.15
	}
}
supensionBridgeMaterial = BuildMaterial {
	shader = "UnlitTintedTex",
	shadercolors={
		_Color = { colorsource="highway" }
	},
	textures={
		_MainTex = "suspensionBridge.png"
	}
}


-- Build Object

-- Objects that are over the track, don't show them when we have a mode with jumping (wakeboard)

if not jumping then 

	-- Set up nodes and assorted tables for the objects

	bridgeNodes = {}
	--bridgeColors = {}
	
	streetsignNodes = {}
	streetsignOffsets ={}
	--streetsignColors = {}
	
	streetsignWideNodes = {}
	--streetsignWideColors = {}
	
	for i=1,#track do
		if (not track[i].funkyrot) then
			if i%80==0 and track[i].trafficstrength < 0.4 and math.random() < 0.2 then 
				bridgeNodes[#bridgeNodes+1] = i
				--bridgeColors[#bridgeColors+1] = track[i].color
			end
			
			if i%130==0 and track[i].trafficstrength > 0.1 then
				streetsignNodes[#streetsignNodes+1] = i	
				streetsignOffsets[#streetsignOffsets+1] = {-trackWidth-9, -4, 0}
				--streetsignColors[#streetsignColors+1] = track[i].color
			end
			
			if i%210==0 and track[i].trafficstrength < 0.4 then
				streetsignWideNodes[#streetsignWideNodes+1] = i	
				--streetsignWideColors[#streetsignWideColors+1] = track[i].color
			end
		end
	end

	-- Bridge, only shown in low traffic (many blocks) areas
	
	CreateObject{
		name="Bridge",
		gameobject={
			visible = false, -- we don't want to see this instance of the object
			mesh="bridge.obj",
			material=objectMaterial
		}
	}

	BatchRenderEveryFrame{
		prefabName="Bridge", 
		locations=bridgeNodes,
		--offsets
		--rotations 
		rotateWithTrack=true,
		--scales 
		--colors = bridgeColors,
		maxShown = iflofi(1, 3),
		maxDistanceShown=10000,
		collisionLayer = 1,
		testAndHideIfCollideWithTrack=false,
	}

	-- Street signs, shown rarely but all over

	CreateObject{
		name="Streetsign",
		gameobject={
			visible = false, -- we don't want to see this instance of the object
			mesh="streetsign.obj",
			material=lightMaterial
		}
	}

	BatchRenderEveryFrame{
		prefabName="Streetsign", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
		locations=streetsignNodes,
		offsets=streetsignOffsets,
		--rotations 
		rotateWithTrack=true,
		--scales 
		--colors = streetsignColors,
		
		maxShown = iflofi(15, 50),
		maxDistanceShown=10000,
		collisionLayer = 1,--will collision test with other batch-rendered objects on the same layer. set less than 0 for no other-object collision testing
		testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
	}


	-- Wide Street signs, shown mostly in low traffic areas

	CreateObject{
		name="StreetsignWide",
		gameobject={
			visible = false, -- we don't want to see this instance of the object
			mesh="streetsignWide.obj",
			material=lightMaterial
		}
	}

	BatchRenderEveryFrame{
		prefabName="StreetsignWide", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
		locations=streetsignWideNodes,
		--offsets
		--rotations 
		rotateWithTrack=true,
		--scales 
		--colors = streetsignWideColors,
		
		maxShown=iflofi(15, 50),
		maxDistanceShown=10000,
		collisionLayer = 1,--will collision test with other batch-rendered objects on the same layer. set less than 0 for no other-object collision testing
		testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
	}

end

-- Suspension Bridge, only shown somtimes

if hasSuspensionBridge then
	suspensionBridgeMesh = BuildMesh{
		mesh="suspensionBridge.obj",
		shapetotrack_at = suspensionBridgePos
	}

	CreateObject{
		name="SuspensionBridge",
		gameobject={
			visible = true,
			pos = track[suspensionBridgePos].pos,
			mesh = suspensionBridgeMesh,
			material=supensionBridgeMaterial
		}
	}
end


-- Buildings, only shown in high traffic areas
-- Regular buildings and Skyscrapers shown during the SkyscraperSegment

buildingNodes = {}
buildingOffsets = {}
buildingScales = {}
--buildingRotations = {}

SkyscraperNodes = {}
SkyscraperOffsets = {}
SkyscraperScales = {}
SkyscraperRotations = {}

for i=1,#track do

   --if i%5==0 and track[i].intensity > 0.6 and not onlyRegularRotScales[i].x == 0 and track[i].trafficstrength + track[i].intensity > 0.6 then
   if i%5==0 and track[i].intensity > 0.6 and track[i].trafficstrength + track[i].intensity > 0.6 and not track[i].funkyrot then
	
		
		-- Are we outside the skyspscraper segment?
		if (not hasSkyscrapers or SkyscraperSegments[i] == 0) then
			buildingNodes[#buildingNodes+1] = i	
			local xOffset = trackWidth + 20 + 5*math.random()
			if i%2 == 0 then xOffset = xOffset * -1 end
			buildingOffsets[#buildingOffsets+1] = {xOffset,-4,0}		
			buildingScales[#buildingScales+1] = {0.8+math.random()*0.5, 1+(math.random()+track[i].trafficstrength+track[i].intensity)*0.1, 0.8+math.random()*0.5}
			--buildingRotations[#buildingRotations+1] = { 0, math.random()*360, 0 }
		else
			SkyscraperNodes[#SkyscraperNodes+1] = i	
			local xOffset = trackWidth + 25 + 25*math.floor(math.random()*2)
			if i%2 == 0 then xOffset = xOffset * -1 end
			SkyscraperOffsets[#SkyscraperOffsets+1] = {xOffset,-4,0}		
			SkyscraperScales[#SkyscraperScales+1] = {0.8+math.random()*0.5, 1+(math.random()+track[i].trafficstrength+track[i].intensity)*0.1, 0.8+math.random()*0.5}
			SkyscraperRotations[#SkyscraperRotations+1] = { 0, math.floor(math.random()*4)*90, 0 }
		end		
	end
end

CreateObject{
	name="Building",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="building.obj",
		material=objectMaterial
	}
}

BatchRenderEveryFrame{
	prefabName="Building", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations = buildingNodes,
	offsets=buildingOffsets,
	--rotations =buildingRotations,
	rotateWithTrack=true,
	scales = buildingScales,
	--colors = buildingColors,
	
	maxShown=iflofi(20, 80),
	maxDistanceShown=10000,
	collisionLayer = 1,--will collision test with other batch-rendered objects on the same layer. set less than 0 for no other-object collision testing
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}

CreateObject{
	name="Skyscraper",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="Skyscraper.obj",
		material=lightMaterial
	}
}

BatchRenderEveryFrame{
	prefabName="Skyscraper", --tell the game to render these prefabs in a batch (with Graphics.DrawMesh) every frame
	locations = SkyscraperNodes,
	offsets=SkyscraperOffsets,
	rotations = SkyscraperRotations,
	rotateWithTrack=true,
	scales = SkyscraperScales,
	--colors = buildingColors,
	
	maxShown=iflofi(20, 80),
	maxDistanceShown=2000,
	collisionLayer = 1,--will collision test with other batch-rendered objects on the same layer. set less than 0 for no other-object collision testing
	testAndHideIfCollideWithTrack=false --if true, it checks each render location against a ray down the center of the track for collision. Any hits are not rendered
}



----------------------------------------------------
-- Repeated Mesh Rails
-- These objects are shown alongside the track

-- Set up base tables

guardrailScales ={}
streetlightOffsets ={}
guardrailOffsets ={}
for i=1,#track do
	-- Streetlights appear all the time, except during a bridge segment
	-- Guardrails appear only during funky rot, but not during a bridge segment
	if (suspensionBridgeNodes[i] == 1) then
		guardrailScales[i] = { 0, 0, 0 }
	
	elseif (onlyFunkyRotScales[i].x == 1) then
		guardrailScales[i] = { 1, 1, 1 }
		
	else	
		guardrailScales[i] = { 0, 0, 0 }
	end	
	
	streetlightOffsets[i] = {trackWidth+2.25, -4, 0}	
	guardrailOffsets[i] = {-trackWidth-2, 0, 0}
end


-- Power Lines, shown continously except for funky rot

powerLinePrefabLength = 160;

CreateObject{
	name="PowerLinesPrefab",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="PowerLines.obj",
		material=objectMaterial
	}
}

CreateRepeatedMeshRail{
	prefabName="PowerLinesPrefab",
   ahead_renderdist = iflofi(350, 1500),
   behind_renderdist = 20,
   spacing = 0, -- default is 0, for a seamless repeated mesh track. spacing can also be accomplished in the OBJ file by not starting at right at the origin, but further down the positive z axis
	colorMode = "static", --"highway" --"highwayinverted"
   track_tilt_factor = 1, -- 0-1 value. Default is 1. set zero here to have the poles always stick straight up instead of following the hill
   scales = onlyRegularRotScalesSoftY,

   buildlive = false, -- if live mode is not used it will take longer to load and use more memory, but it will run better during gameplay
   --maxmeshesperbatch = 4, -- when in live mode, this is the number of mesh copies that can be shaped to the track in a single frame without a hiccup
   calculatenormals = false --set true if using lighting
}



-- Variations shown in addition, only shown every few power lines
-- These work because the objects have the same z-axis length as the powerlines

CreateObject{
	name="PowerLinesVar1",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="powerLines_Var1.obj",
		material=objectMaterial
	}
}

CreateRepeatedMeshRail{
	prefabName="PowerLinesVar1",
   ahead_renderdist = iflofi(350, 1500),
   behind_renderdist = 20,
   spacing = powerLinePrefabLength * 7,
	colorMode = "static", --"highway" --"highwayinverted"
   track_tilt_factor = 1,
   scales = onlyRegularRotScalesSoftY,

   buildlive = false,
   calculatenormals = false 
}



-- Variations shown in addition, only shown every few power lines
-- These work because the objects have the same z-axis length as the powerlines

CreateObject{
	name="PowerLinesVar2",
	gameobject={
		visible = false,
		mesh="powerLines_Var2.obj",
		material=objectMaterial
	}
}

CreateRepeatedMeshRail{
	prefabName="PowerLinesVar2",
   ahead_renderdist = iflofi(350, 1500),
   behind_renderdist = 20,
   spacing = powerLinePrefabLength * 5, 
	colorMode = "static", --"highway"
   track_tilt_factor = 1,
   scales = onlyRegularRotScalesSoftY,

   buildlive = false, 
   calculatenormals = false 
}


-- Street lights, show them all the time


CreateObject{
	name="StreetlightPrefab",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="streetlight.obj",
		material=lightMaterial
	}
}

CreateRepeatedMeshRail{
	prefabName="StreetlightPrefab",
   ahead_renderdist = iflofi(350, 1500),
   behind_renderdist = 20,
   spacing = 60, -- default is 0, for a seamless repeated mesh track. spacing can also be accomplished in the OBJ file by not starting at right at the origin, but further down the positive z axis
	colorMode = "static", --"highway" --"highwayinverted"
   track_tilt_factor = 1, -- 0-1 value. Default is 1. set zero here to have the poles always stick straight up instead of following the hill
   offsets = streetlightOffsets,
   --scales = streetlightScales,

   buildlive = false, -- if live mode is not used it will take longer to load and use more memory, but it will run better during gameplay
   --maxmeshesperbatch = 4, -- when in live mode, this is the number of mesh copies that can be shaped to the track in a single frame without a hiccup
   calculatenormals = false --set true if using lighting
}


-- Guard rails, show them only during funky rot

CreateObject{
	name="GuardrailPrefab",
	gameobject={
		visible = false, -- we don't want to see this instance of the object
		mesh="guardrail.obj",
		material=objectMaterial
	}
}

CreateRepeatedMeshRail{
	prefabName="GuardrailPrefab",
   ahead_renderdist = iflofi(350, 1500),
   behind_renderdist = 20,
   spacing = 0, -- default is 0, for a seamless repeated mesh track. spacing can also be accomplished in the OBJ file by not starting at right at the origin, but further down the positive z axis
	colorMode = "static", --"highway" --"highwayinverted"
   track_tilt_factor = 1, -- 0-1 value. Default is 1. set zero here to have the poles always stick straight up instead of following the hill
   offsets = guardrailOffsets,
   scales = guardrailScales,

   buildlive = false, -- if live mode is not used it will take longer to load and use more memory, but it will run better during gameplay
   --maxmeshesperbatch = 4, -- when in live mode, this is the number of mesh copies that can be shaped to the track in a single frame without a hiccup
   calculatenormals = false --set true if using lighting
}

